;; This transformation mimics Griswold's Transformation of Parnas's KWIC
;; program form a function-oriented architecture to a data-oriented
;; architecture.
;; Reference : W. G. Griswold, 
;; Program Restructuring as an Aid to Software Maintenance,
;; Ph.D. Thesis, Technical Report 91-08-04, 
;; Department of Computer Science and Engineering,
;; University of Washington, July 1991. 

;; The haskell initial program differs from Griswold's Scheme implementation.
;; The main differences are :
;;  - there are no references to global mutable variables (no state monad),
;;  - modules are syntactically defined.

;; Remark : In the data-oriented architecture, it would be better to aim at 
;;  an abstract data-type for line storages.

;; 1- Move out function insline (p.171)
(haskell-refac-liftDefToTopLevelIn "putfile" "insline" "Input")
(haskell-refac-moveDefBetweenModules "insline" "Input" "LineStorage" )

;; 2- Extract function lines (p.171)
(haskell-refac-newDefIdentAppIn "numlines" "length" "1" "ls_lines" "Shifter")
(haskell-refac-liftDefToTopLevelIn "cssetup" "ls_lines" "Shifter")
(haskell-refac-moveDefBetweenModules "ls_lines" "Shifter" "LineStorage" )

;; 3- Extract function words (p.171) & fold its occurences (p.178)
(haskell-refac-newDefIdentAppIn "numwords" "length" "1" "ls_words" "Shifter")
(haskell-refac-liftDefToTopLevelIn "cssetup" "ls_words" "Shifter")
(haskell-refac-foldToplevelDefinition "ls_words" "Shifter")
(haskell-refac-moveDefBetweenModules "ls_words" "Shifter" "LineStorage" )

;; 4- The empty line storage is abstracted (instead of empty list)
(haskell-refac-newDefIdentIn "putfile" "emptylist" "emptyStorage" "Input")
(haskell-refac-liftDefToTopLevelIn "putfile" "emptyStorage" "Input")
(haskell-refac-moveDefBetweenModules "emptyStorage" "Input" "LineStorage" )

;; 5- move csline_inf and expose csword & cswords (p.173)
(haskell-refac-liftDefToTopLevelIn "alph" "csline_inf" "Alphabet")
(haskell-refac-moveDefBetweenModules "csline_inf" "Alphabet" "Shifter" ) 
(haskell-refac-liftDefToTopLevelIn "csline_inf" "csword" "Shifter")
(haskell-refac-liftDefToTopLevelIn "csline_inf" "cswords" "Shifter")

;; 6- Extract cslines (p.173)
(haskell-refac-newDefIdentAppIn "numitems" "snd" "1" "cslines" "Alphabet")
(haskell-refac-liftDefToTopLevelIn "alph" "cslines" "Alphabet")
(haskell-refac-moveDefBetweenModules "cslines" "Alphabet" "Shifter" )

;; 7- Extract cslines (p.173)
(haskell-refac-newDefInfixAppIn "alph_aux" "!" "csline" "Alphabet")
(haskell-refac-liftDefToTopLevelIn "alph" "csline" "Alphabet") 
(haskell-refac-moveDefBetweenModules "csline" "Alphabet" "Shifter" ) 

;; 8- Rename csline and move it to the shift module (p.175)
(haskell-refac-renameLocalIdentIn "csline" "Output" "printable_csline") 
(haskell-refac-liftDefToTopLevelIn "allalphcslines" "printable_csline" "Output") 
(haskell-refac-moveDefBetweenModules "printable_csline" "Output" "Shifter" )

;; 9- Extract alphcsline (p.175)
(haskell-refac-newDefInfixAppIn "allalphcslines_aux" "!" "alphcsline" "Output")
(haskell-refac-liftDefToTopLevelIn "allalphcslines" "alphcsline" "Output")
(haskell-refac-moveDefBetweenModules "alphcsline" "Output" "Alphabet" )

;; 10- Extract alphcslines (p.175)
(haskell-refac-newDefIdentAppIn "numcslines" "snd" "1" "alphcslines" "Output")
(haskell-refac-liftDefToTopLevelIn "allalphcslines" "alphcslines" "Output")
(haskell-refac-moveDefBetweenModules "alphcslines" "Output" "Alphabet" )

;; 11- Fold the occurences of line (p.178)
;; Some syntactic occurence do not match the intended use of line.
;; These instances are unfolded to revert to the original code
;; (the state of the program is temporary not satisfactory before the unfolding).
(haskell-refac-newDefIdentAppToIn "csword" "nth" "2" "ls" "line" "Shifter")
(haskell-refac-liftDefToTopLevelIn "csword" "line" "Shifter") 
(haskell-refac-foldToplevelDefinition "line" "Shifter")
(haskell-refac-foldToplevelDefinition "line" "Shifter") ;; a second time to fold nested occurences
(haskell-refac-unfoldIn "line" "printable_csline" "Shifter")
(haskell-refac-unfoldIn "line" "csword" "Shifter")
(haskell-refac-moveDefBetweenModules "line" "Shifter" "LineStorage" )


;; 12- Move out allwords (p.179)
(haskell-refac-liftDefToTopLevelIn "cssetup" "allwords" "Shifter")
(haskell-refac-moveDefBetweenModules "allwords" "Shifter" "LineStorage"  )
